%% Init
close  all
clc
clear  variables

figNum = 1;         % Figure number-counter
PSI_r = 30;         % Reference angle for simulation
sim_t = 500;        % Simulation time

% \\\ Predetermined constants
addpath('Data-files')               % Add folder for .mat-files
load('constants_5.2.mat')           % K_w, lambda, omega_0, sigma
load('constants_5.3.mat')           % K, K_pd, T, T_d, T_f, w_c

% \\\ X and Y output of ship 
load('north_east_5.3.mat')          % x1, x2, x3, y1, y2, y3

% \\\ Rudder input to ship 
load('rudder_input_5.3.mat')         % delta1, delta2, delta3, t1, t2, t3

% \\\ Psi output of ship 
load('compass_measurment_5.3.mat')   % psi1, psi2, psi3

% \\\ Add working path for simulink models
addpath('Simulink models tasks')
%

%% Step 1, discretization of sytsem
% \\\ System matrices
A = [0 1 0 0 0; -omega_0^2 -2*lambda*omega_0 0 0 0; 0 0 0 1 0; ...  
    0 0 0 -1/T -K/T; 0 0 0 0 0];
B = [0; 0; 0; K/T; 0];
C = [0 1 1 0 0];
D = 0;
E = [0 0; K_w 0; 0 0; 0 0; 0 1];

% \\\ Sample frequency
F_s = 100; 
T_s = 1/F_s;

% \\\ Zero-order-Hold discretization
[Ad, Bd]  = c2d(A,B,T_s);    Cd = C;
[Ad, Ed] = c2d(A,E,T_s);    Dd = D;
%

%% Step 2, Estimation of var(v)
load_system('task5_5_b.slx')
sim('task5_5_b.slx')

% \\\ R is the Measurment noise variance
R = var(sim_compass*pi/180);
%

%% Step 3, Discrete Kalman Filter
% \\\ Q is the Process noise covariance
Q = [30 0; 0 10^(-6)];
P_0 = [1 0 0 0 0; 0 0.013 0 0 0; 0 0 pi^2 0 0; 0 0 0 1 0; ...
    0 0 0 0 2.5*10^-4];
X_0 = [0; 0; 0; 0; 0];
R = R/T_s;
I = diag([1 1 1 1 1]);

% \\\ Put data in a struct for use in the Kalman filter
data = struct('Ad',Ad,'Bd',Bd,'Cd',Cd,'Ed', Ed, 'Q',Q,'R', R,'P_0',P_0, ...
              'X_0',X_0, 'I', I);
%

%% Step 4, Feed forward estimated bias
load_system('task5_5_d.slx')
sim('task5_5_d.slx')

% \\\ Plot measured compass with and without estimated bias
figure(figNum)
figNum = figNum+1;
plot(t,sim_PSI_r*ones(1,length(t)),'r--', t2, psi2, 'b', t, sim_compass, 'm')
xlabel('t [s]','FontSize', 20); ylabel('Angle [deg]','FontSize', 20);
legend({'$\psi_{r}$', '$\psi + v$ (without $\hat{b}$)', ...
    '$\psi + v$ (with $\hat{b}$)'}, 'FontSize', 24, ...
    'Interpreter', 'latex', 'Location', 'SouthEast')
grid  on; axis([0 500 0 40])

%% Step 5, Feed forward estimated bias and wave filtered psi
load_system('task5_5_e.slx')
sim('task5_5_e.slx')

% \\\ Plot measured compass and estimated compass
figure(figNum)
figNum = figNum+1;
plot(t,sim_PSI_r*ones(1,length(t)), 'r--', t, sim_compass, 'b', t, psi_filtered, 'm')
xlabel('t [s]','FontSize', 20); ylabel('Angle [deg]','FontSize', 20);
legend({'$\psi_{r}$', '$\psi + \psi_{w} + v$' ...
    , '$\hat{\psi}$'}, 'FontSize', 24, ...
    'Interpreter', 'latex', 'Location', 'best')
ax = gca; grid on; axis([0 500 -5 40])

% \\\ Plot measured psi with and without Kalman filtered bias and waves
figure(figNum)
figNum = figNum+1;
plot(t,sim_PSI_r*ones(1,length(t)), 'r--', t3, psi3, 'b', t, sim_compass, 'm')
xlabel('t [s]','FontSize', 20); ylabel('Angle [deg]','FontSize', 20);
legend({'$\psi_{r}$', 'Without KF' ...
    'With KF'}, 'FontSize', 24,...
    'Interpreter', 'latex', 'Location', 'best')
ax = gca; grid on; axis([0 500 -5 40])

% \\\ Plotting rudder input delta
figure(figNum)
figNum = figNum+1;
plot(t, b_filtered,'m', t,delta)
xlabel('t [s]','FontSize', 20); ylabel('Angle [deg]','FontSize', 20);
legend({'$\hat{b}$', '$\delta$'}, 'FontSize', 24, ...
    'Interpreter', 'latex', 'Location', 'best')
ax = gca; grid on;

% \\\ Wave influence (current turned off, delta = 0)
load_system('task5_5_e_2.slx')
sim('task5_5_e_2.slx')

% \\\ Plotting wave influence on system
figure(figNum)
figNum = figNum+1;
plot(t(1:length(sim_compass)), sim_compass, t(1:length(psi_filtered)), psi_filtered)
xlabel('t [s]','FontSize', 20); ylabel('Angle [deg]','FontSize', 20);
legend({'$\psi_{w}$', '$\hat{\psi}_{w}$'}, 'FontSize', 24, ...
    'Interpreter', 'latex', 'Location', 'best')
ax = gca; grid on;
axis([0 500 -4 4]);
%